title: Lazy Loading Images on the Web
url: http://developer.telerik.com/featured/lazy-loading-images-on-the-web/
hash_url: 911d8d2cce04195fe21b4efeaa0e25cb

<p>Images on the web offer a bit of a conundrum. They are often what makes a web page feel vibrant and interesting, but they can also dramatically hurt web page performance by adding a significant amount of weight to a page.</p>
<p>On this site, we use a lot of images within our content. As the person who maintains this site, I do my best to optimize them, but in many cases the sum total of the image weight on a page is still significant (especially as we’ve come to rely on animated GIFs for illustrations). The thing is, much of this image weight sits well offscreen. As much as I’d like to believe that every visitor reads every word we write, it’s highly likely that many visitors are downloading images that they simply do not ever see.</p>
<h2>The Page Weight Problem</h2>
<p>This issue isn’t unique to this site. <a href="http://mobile.httparchive.org/interesting.php?a=All&amp;l=Jul%201%202015">According to HTTPArchive</a>, images now account for 63% of page weight. As developers, we work hard to optimize and minify our JavaScript code and CSS to make them as small as possible, but, by sheer volume, that can never have the impact of simply reducing image weight.</p>
<p>Consider that, as <a href="http://www.soasta.com/blog/page-bloat-average-web-page-2-mb/">Tammy Everts notes</a>, “images alone comprise more page weight (1310 KB) than an entire page in May 2012 (1059 KB), just three years ago.” It’s a problem for any visitor with a data cap – which is pretty much everyone, especially internationally. Images your visitors never see are costing them money.</p>
<p>And the page weight issue is only getting worse:</p>
<blockquote><p>As of May [2015], the average web page surpassed the 2 MB mark. This is almost twice the size of the average page just three years ago. – <a href="http://www.soasta.com/blog/page-bloat-average-web-page-2-mb/">source</a></p>
</blockquote>
<p>So, what can we do?</p>
<p>Well, one option is to “lazy load” images. In essence, we use a little bit of JavaScript to determine which images are in (or near) the viewport and only download images that the user will likely see. It is a strategy that is not without its own flaws (some of which are covered in this <a href="http://www.smashingmagazine.com/2015/02/redefining-lazy-loading-with-lazy-load-xt/">article</a> by the author of one lazy loading library). However, if your site relies on image-heavy content, it is a strategy worth considering – especially when targeting mobile devices.</p>
<p>In the remainder of this article, we’ll look at a number of different solutions for implementing lazy loading of images.</p>
<h3>Sample Page</h3>
<p>To assist us in reviewing the options for lazy loading images, I’ve built a simple example web page. The page is designed as a “Teen Titans Go” fan page that lists out a large number of the characters, with their images.</p>
<p><a href="http://developer.telerik.com/wp-content/uploads/2015/10/samplepage.jpg"><img src="http://developer.telerik.com/wp-content/uploads/2015/10/samplepage.jpg" alt="Teen Titans" width="800" height="523" class="alignnone size-full wp-image-60881"/></a></p>
<p>I rebuilt the page using a variety of solutions to compare how they are implemented and how they work. The full code for all the variations can be found <a href="https://github.com/remotesynth/Lazy-Loading-Images">on GitHub</a>.</p>
<h2>Custom Solution</h2>
<p>At its most basic implementation, building a custom solution for lazy loading images is not complicated. Here’s what we need to do:</p>
<ol>
<li>Build the HTML so that images are not automatically loaded (this is typically done by specifying the actual <code>src</code> in a <a href="https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/Using_data_attributes">data attribute</a>);</li>
<li>Watch changes to the viewport or scrolling to see which images have or may soon enter the viewport;</li>
<li>Swap the data attribute and the <code>src</code> so that the image is loaded.</li>
</ol>
<p>For the first item, you might be concerned that an <code>img</code> tag without a <code>src</code> is not valid HTML. Unfortunately, it appears that you cannot place the actual source in the <code>src</code> attribute while somehow preventing the browser from loading the image using JavaScript. You can place a “dummy” source image, however, such as a spacing or loading gif.</p>
<p>Let’s start by seeing how to build this in plain JavaScript.</p>
<h3>Plain JavaScript</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/custom.html">View this demo</a></p>
<p>The first thing is to make sure the images do not load by specifying the image source using a <code>data-src</code> attribute rather than placing it in the <code>src</code>. In our simple example, we are not going to worry about setting a dummy <code>src</code> or loading image.</p>
<pre><code class="language-markup">&lt;img data-src="images/Robin.jpg" alt="" /&gt;</code></pre>
<p>I should note that you may want to add a width and height to your images to ensure that they take up the appropriate space. For our sample that isn’t completely necessary.</p>
<p>Now we need a way to test if an image is within the viewport. Thankfully, with support for <code>getBoundingClientRect</code> being <a href="http://caniuse.com/#feat=getboundingclientrect">nearly universal</a>, there is a reliable method to do that. The function we are using is borrowed from a <a href="http://stackoverflow.com/questions/123999/how-to-tell-if-a-dom-element-is-visible-in-the-current-viewport/7557433#7557433">response on StackOverflow</a>.</p>
<pre><code class="language-javascript">function isElementInViewport (el) {&#13;
&#13;
    var rect = el.getBoundingClientRect();&#13;
&#13;
    return (&#13;
        rect.top &gt;= 0 &amp;&amp;&#13;
        rect.left &gt;= 0 &amp;&amp;&#13;
        rect.bottom &lt;= (window.innerHeight || document.documentElement.clientHeight) &amp;&amp; /*or $(window).height() */&#13;
        rect.right &lt;= (window.innerWidth || document.documentElement.clientWidth) /*or $(window).width() */&#13;
    );&#13;
}</code></pre>
<p>To use this function, we’d pass an image or an image container and the function will return true if the passed element is within the viewport, or false if it is not.</p>
<p>Next, we need to get all of the images we want to lazy load and then test them against this function every time a scroll or change to the viewport occurs. Let’s look at the full code and then examine what it is doing.</p>
<pre><code class="language-javascript">//these handlers will be removed once the images have loaded&#13;
window.addEventListener("DOMContentLoaded", lazyLoadImages);&#13;
window.addEventListener("load", lazyLoadImages);&#13;
window.addEventListener("resize", lazyLoadImages);&#13;
window.addEventListener("scroll", lazyLoadImages);&#13;
&#13;
function lazyLoadImages() {&#13;
  var images = document.querySelectorAll("#main-wrapper img[data-src]"),&#13;
      item;&#13;
  // load images that have entered the viewport&#13;
  [].forEach.call(images, function (item) {&#13;
    if (isElementInViewport(item)) {&#13;
      item.setAttribute("src",item.getAttribute("data-src"));&#13;
      item.removeAttribute("data-src")&#13;
    }&#13;
  })&#13;
  // if all the images are loaded, stop calling the handler&#13;
  if (images.length == 0) {&#13;
    window.removeEventListener("DOMContentLoaded", lazyLoadImages);&#13;
    window.removeEventListener("load", lazyLoadImages);&#13;
    window.removeEventListener("resize", lazyLoadImages);&#13;
    window.removeEventListener("scroll", lazyLoadImages);&#13;
  }&#13;
}</code></pre>
<p>The first thing we do is ensure that we are watching for scrolls and changes to the viewport by listening on the <code>DOMContentLoaded</code>, <code>load</code>, <code>resize</code> and <code>scroll</code> events. Every time one of these events occurs, we call a method to check if any images have entered the viewport. (If you’re concerned about how often these events will be called, I discuss that issue in a later section.)</p>
<p>Looking at the <code>lazyLoadImages</code> method, we first get all the images that have not yet loaded. We do this by selecting only those that still have a <code>data-src</code> attribute. (As we’ll see in upcoming examples, there are a number of methods to do this, but, honestly, I have not tested which method is more performant. However, the performance implication of DOM node retrieval is negligible for the overwhelming majority of cases)</p>
<p>If the image has entered the viewport, we swap the value of the <code>data-src</code> attribute with the <code>src</code> attribute and remove the <code>data-src</code> attribute. Finally, if there are no images left unloaded, we simply remove the event listeners.</p>
<p>All in all, I’d say this was pretty simple, though it’s a very limited implementation. You could expand upon this to test for images sitting just outside the viewport so that there would be a potentially smaller visibility delay for the user – although this only really helps if the user manually scrolls.</p>
<h3>jQuery</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/custom-jquery.html">View this demo</a></p>
<p>If you use jQuery on your site, you can save a few lines of code. For instance, we can move turning the event handlers on and off into a single line of code. However, all in all, we save only a handful lines of code.</p>
<pre><code class="language-javascript">$(window).on('DOMContentLoaded load resize scroll', function () {;&#13;
  var images = $("#main-wrapper img[data-src]");&#13;
  // load images that have entered the viewport&#13;
  $(images).each(function (index) {&#13;
    if (isElementInViewport(this)) {&#13;
      $(this).attr("src",$(this).attr("data-src"));&#13;
            $(this).removeAttr("data-src");&#13;
    }&#13;
  })&#13;
  // if all the images are loaded, stop calling the handler&#13;
  if (images.length == 0) {&#13;
    $(window).off('DOMContentLoaded load resize scroll')&#13;
  }&#13;
})&#13;
&#13;
// source: http://stackoverflow.com/questions/123999/how-to-tell-if-a-dom-element-is-visible-in-the-current-viewport/7557433#7557433&#13;
function isElementInViewport (el) {&#13;
    var rect = el.getBoundingClientRect();&#13;
&#13;
    return (&#13;
        rect.top &gt;= 0 &amp;&amp;&#13;
        rect.left &gt;= 0 &amp;&amp;&#13;
        rect.bottom &lt;= $(window).height() &amp;&amp;&#13;
        rect.right &lt;= $(window).width()&#13;
    );&#13;
}</code></pre>
<p>If you go the build your own solution route, choosing straight JavaScript versus jQuery ends up being just a matter of personal preference.</p>
<h3>Problems with These Solutions</h3>
<p>Right now, we’ve just implemented extremely basic lazy loading functionality for images. A more robust solution would handle setting an offset, whereby elements just off screen are loaded as well as those on screen. Also, while the performance of our script may be suitable for a page with a relatively limited number of images (and, in this case, the images are not huge), its performance would likely degrade significantly for a more complex page with more images as the event handler is constantly being called and looping through a list of images.</p>
<p>Some additional features might be nice too, for instance having success and failure handlers for the loading process could prove useful. It’d also be nice to implement things like <code>srcset</code> and <code>picture</code> for responsive images. Lastly, I might want to add support for a  loading images in case the image file isn’t fully downloaded when it enters the viewport. I might even, perhaps, want some sort of animation or transition when images appear.</p>
<p>The good news is that there are a lot of pre-built libraries for handling lazy loading of images available that do many of these things. Let’s look at some of them.</p>
<h2>Libraries</h2>
<p>For the most part, as you look through the library variations below, you may notice that they all function very similarly. For our sample, the implementation looks surprisingly similar across the board. Choosing one comes down to whether you want to require jQuery (most do) and what kind of options you need, as some offer far more configuration than others.</p>
<h3>LazyLoad</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/lazyload.html">View this demo</a></p>
<p>One of the first lazy loading image libraries was the <a href="http://www.appelsiini.net/projects/lazyload">Lazy Load Plugin for jQuery</a>. It inspired a number of additional libraries including <a href="http://verlok.github.io/lazyload/">LazyLoad</a>.</p>
<p>One difference between LazyLoad and the other solutions is that the images use <code>data-original</code> for the source rather than <code>data-src</code>.</p>
<pre><code class="language-markup">&lt;img data-original="images/GreenLantern.jpg" alt="" width="374" height="260" /&gt;</code></pre>
<p>Once our images are set up, lazy loading them is incredibly simple. Just include the JavaScript file (obviously) and initialize LazyLoad. In our sample code below, we’ve added a threshold of 50 pixels, meaning that items just off-screen will also be loaded. We’ve also added a success handler.</p>
<pre><code class="language-javascript">var myLazyLoad = new LazyLoad({&#13;
  threshold: 50,&#13;
  callback_load: function(e) {&#13;
    console.log($(e).attr("data-original") + " loaded" );&#13;
  }&#13;
});</code></pre>
<p>The image below shows scrolling through the mobile version of the page loading one image at a time. In the console, each image indicates that it has successfully loaded.</p>
<p><a href="http://developer.telerik.com/wp-content/uploads/2015/10/LazyLoad_opt.gif"><img src="http://developer.telerik.com/wp-content/uploads/2015/10/LazyLoad_opt.gif" alt="LazyLoad_opt" width="400" height="533" class="alignnone size-full wp-image-60891"/></a></p>
<p>LazyLoad supports a number of <a href="http://verlok.github.io/lazyload/#options">additional options</a>, which we won’t cover here.</p>
<h3>bLazy.js</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/blazy.html">View this demo</a></p>
<p><a href="http://dinbror.dk/blazy/">bLazy</a> (or [Be]Lazy) is a relatively recent library that aims to offer a number of key features while remaining small and lightweight. One significant difference of bLazy is that it does not require jQuery.</p>
<p>To identify which images are to be lazy loaded, by default bLazy requires you to add a CSS class onto each. In this example, I am using the default CSS class, but the <a href="http://dinbror.dk/blog/blazy/#Offset">selector can be customized</a>.</p>
<pre><code class="language-markup">&lt;img data-src="images/Batgirl.jpg" alt="" width="374" height="260" class="b-lazy" /&gt;</code></pre>
<p>After that, you simply include and initialize the script. Below I have done that while also setting an offset and a success and error handler.</p>
<pre><code class="language-javascript">var bLazy = new Blazy({&#13;
    offset: 50,&#13;
    success: function(e){&#13;
        console.log($(e).attr("src") + " loaded" );&#13;
  },&#13;
    error: function(ele, msg){&#13;
        console.log(msg)&#13;
  }&#13;
});</code></pre>
<p>Below you can see multiple images loading at a time as the page is scrolled.</p>
<p><a href="http://developer.telerik.com/wp-content/uploads/2015/10/blazy_opt.gif"><img src="http://developer.telerik.com/wp-content/uploads/2015/10/blazy_opt.gif" alt="blazy_opt" width="600" height="401" class="alignnone size-full wp-image-60901"/></a></p>
<p>bLazy supports a number of <a href="http://dinbror.dk/blog/blazy/#Options">options</a> including the ability to serve smaller or larger images based upon screen size. Check the <a href="http://dinbror.dk/blog/blazy/">documentation</a> for more details.</p>
<h3>Unveil</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/unveil.html">View this demo</a></p>
<p><a href="http://luis-almeida.github.io/unveil/">Unveil</a> is another script inspired by <a href="http://www.appelsiini.net/projects/lazyload">jQuery_lazyload</a>, so, it does require jQuery. It is very lightweight, however, being less than 1k.</p>
<p>One nice thing about Unveil is that it does not require any special markup on your images beyond the <code>data-src</code> attribute.</p>
<pre><code class="language-markup">&lt;img data-src="images/WonderTwins.jpg" alt="" /&gt;</code></pre>
<p>This is because we specify the images that will be supplied to Unveil via a jQuery selector. Simply load the script and apply Unveil to our selected images on document ready.</p>
<pre><code class="language-javascript">$(function () {&#13;
  $("#main-wrapper img").unveil(50, function() {&#13;
    $(this).load(function() {&#13;
      console.log(this.src + " loaded");&#13;
    });&#13;
  });&#13;
})</code></pre>
<p>As with the prior examples, we’ve also initialized Unveil with an offset of 50 pixels and a success handler.</p>
<p>Unveil is more lightweight than the other libraries and, because of this, doesn’t offer quite as many options as the others discussed. Check <a href="http://luis-almeida.github.io/unveil/">the documentation</a> for additional usage options.</p>
<h3>Lazy Load XT</h3>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/lazyloadxt.html">View this demo</a></p>
<p><a href="https://github.com/ressio/lazy-load-xt">Lazy Load XT</a> probably offers the most features of any of the libraries discussed here. In their <a href="http://www.smashingmagazine.com/2015/02/redefining-lazy-loading-with-lazy-load-xt/">Smashing Magazine article</a>, the authors described their aim to fix many of the deficiencies in the other available libraries.</p>
<p>Lazy Load XT splits itself into two main libraries – the first has the core features and the second adds some of the more advanced options. It also offers additional scripts as plugins and CSS files for effects. For our sample, we only need the basic script file.</p>
<pre><code class="language-markup">&lt;script src="js/jquery.lazyloadxt.js"&gt;&lt;/script&gt;</code></pre>
<p>Just as with Unveil previously, our images only need a <code>data-src</code> attribute. There is no need for any special classes or additional data attributes.</p>
<pre><code class="language-markup">&lt;img data-src="images/BeastBoy.jpg" alt="" /&gt;</code></pre>
<p>Now, all we need to do is initialize Lazy Load XT. In the below code, we are also adding success and error handlers by listening for specific events that LazyLoad emits.</p>
<pre><code class="language-javascript">$(window).lazyLoadXT();&#13;
$(window).on("lazyload", function(e){&#13;
    console.log(e.target.src + " loaded");&#13;
});&#13;
$(window).on("lazyerror", function(e){&#13;
    console.log("Error loading " + e.target.src);&#13;
});</code></pre>
<p>Lazy Load XT supports a lot of <a href="https://github.com/ressio/lazy-load-xt#options">options</a> and <a href="https://github.com/ressio/lazy-load-xt#events">events</a>. I should note, however, that I had some difficulty getting certain options to work easily. While Lazy Load XT has a lot of examples and documentation, it seems (to me anyway) to be missing the simple examples, such as using some of the basic events and configuration options. I also, unfortunately, had some difficulty even getting the basic effects to work (in theory, this was simply a matter of including the correct CSS).</p>
<p>On a more positive note, Lazy Load XT has extensive support for responsive images. It even offers support for lazy loading videos and iframes, making it by far the most comprehensive lazy loading library I came across. Someone has even created a <a href="https://wordpress.org/plugins/lazy-load-xt/">WordPress plugin</a> to lazy load images included within WordPress posts.</p>
<h2>Telerik Platform – Responsive Images Service</h2>
<p><a href="https://rawgit.com/remotesynth/Lazy-Loading-Images/master/telerik.html">View this demo</a></p>
<p>Let’s look at one more example. Within the <a href="http://www.telerik.com/platform">Telerik Platform</a>‘s Backend Services offering is a <a href="http://docs.telerik.com/platform/backend-services/javascript/responsive-images/introduction">responsive images service</a>. It offers:</p>
<ul>
<li>The ability to specify image dimensions for individual images;</li>
<li>Automatic resizing of images based on the target container dimensions.</li>
</ul>
<p>The main benefit for our purposes is that using this service the images are automatically responsive. For example, we can place a huge image in Backend Services but the service will automatically serve the appropriate image based on the device’s screen size and pixel density — no need to use srcset/picture, or to upload ten versions of the images.</p>
<p>For a full overview of the responsive images offering, check out <a href="http://developer.telerik.com/featured/responsive-images-theres-service/">this article by Hristo Borisov</a>.</p>
<p>In addition, Backend Services offer the ability to store and serve files from the CDN. While this isn’t necessary to utilize the responsive images features, it is definitely a plus.</p>
<p><a href="http://developer.telerik.com/wp-content/uploads/2015/10/Platform_files.jpg"><img src="http://developer.telerik.com/wp-content/uploads/2015/10/Platform_files.jpg" alt="Platform_files" width="800" height="467" class="alignnone size-full wp-image-60911"/></a></p>
<p>After uploading the files, we need to right click the file name (i.e. the Batgirl.jpg link in the screenshot above) to get its URL (see <a href="http://docs.telerik.com/platform/backend-services/javascript/files/files-getting-started#obtain-the-file-download-url">the documentation</a> if you need additional details).</p>
<p>In the HTML, we need to swap all the images for the CDN-hosted versions within the <code>data-src</code> attribute. We also need to add a <code>data-responsive</code> attribute to each image. This enables the automatic image replacement – more on that in a moment. For instance, here’s the image tag for Raven (aka my favorite Teen Titan), where the URL follows an <code>/api-key/image-id</code> format.</p>
<pre><code class="language-markup">&lt;img data-src="https://bs2.cdn.telerik.com/v1/h5tc7Cws1Qi9oluI/de916422-6e1f-11e5-a7c8-356526b6da24" alt="" data-responsive /&gt;</code></pre>
<p>We are going to use the JavaScript SDK that Backend Services provides to <a href="http://docs.telerik.com/platform/backend-services/javascript/responsive-images/responsive-images-automatic-use">automatically apply responsive images</a>. So first, let’s include the script.</p>
<pre><code class="language-markup">&lt;script src="https://bs-static.cdn.telerik.com/1.5.6/everlive.all.min.js"&gt;&lt;/script&gt;</code></pre>
<p>Next we need to initialize the service. There are a number of <a href="http://docs.telerik.com/platform/backend-services/javascript/responsive-images/responsive-images-configure-attributes#configuring-html-attributes">additional options</a> that you can specify here, but let’s stick with the defaults.</p>
<pre><code class="language-javascript">var el = new Everlive({&#13;
    apiKey: "your-api-key",&#13;
    scheme: "https"&#13;
});</code></pre>
<p>Since the service is not dependent on jQuery, let’s go ahead and leverage the plain JavaScript version we built earlier. We also need to include the <code>isElementInViewport</code> function shown earlier. After that, it’s just some minor changes to the plain JavaScript code we wrote previously.</p>
<pre><code class="language-javascript">window.addEventListener("DOMContentLoaded", lazyLoadImages);&#13;
window.addEventListener("load", lazyLoadImages);&#13;
window.addEventListener("resize", lazyLoadImages);&#13;
window.addEventListener("scroll", lazyLoadImages);&#13;
&#13;
function lazyLoadImages() {&#13;
  // select only images that do not yet have a src attribute added&#13;
  var images = document.querySelectorAll("#main-wrapper img:not([src])"),&#13;
      item;&#13;
&#13;
  // load images that have entered the viewport&#13;
  [].forEach.call(images, function (item) {&#13;
    if (isElementInViewport(item)) {&#13;
            el.helpers.html.process(item, {}, successHandler, errorHandler);&#13;
    }&#13;
  })&#13;
  // if all the images are loaded, stop calling the handler&#13;
  if (images.length == 0) {&#13;
    window.removeEventListener("DOMContentLoaded", lazyLoadImages);&#13;
    window.removeEventListener("load", lazyLoadImages);&#13;
    window.removeEventListener("resize", lazyLoadImages);&#13;
    window.removeEventListener("scroll", lazyLoadImages);&#13;
  }&#13;
}&#13;
&#13;
function successHandler (e) {&#13;
    console.log("success");&#13;
}&#13;
function errorHandler (e) {&#13;
    console.log(e);&#13;
}</code></pre>
<p>There are a couple of small but important changes to note here. First, we have changed our selector to select any images that do not have a <code>src</code> attribute. The automatic responsive image loading does not like us removing the <code>data-src</code> attribute, but, using this method, we can still select only those images that have not yet been loaded.</p>
<p>Second, if the image is within the viewport, we call <code>el.helpers.html.process()</code> to have the service automatically process them. We’re not passing any options at this time, although we did set success and failure handlers. Since we are loading from an external CDN, it would probably be worth expanding this to support an offset in the future.</p>
<p>There’s many more options available in the responsive images service. If you’re interested in utilizing it, I suggest <a href="http://docs.telerik.com/platform/backend-services/javascript/responsive-images/introduction">reading through the documentation</a> to learn more about what is available.</p>
<h2>Go Further</h2>
<p>Hopefully, by now, you have a good overview of the benefits of lazy loading images and some of the techniques available to you for achieving it. Our samples were intentionally simple, but the options exist to take this to the next level including handling things like responsive images, video, iframes and widgets, depending on the needs of your site.</p>
<p>The important thing to keep in mind here is the cost/benefit to your users. Does the potential for a slight delay in image loading (and, perhaps, a decrease in “scan-ability” of the content) offset the problem of image bloat? Or does your content rely heavily on large imagery that the user may never see, but which may be costing them in terms of loading time or data overages? Is it possibly worth implementing this strategy but targeting specifically mobile, where data can be slow and costly, while ignoring desktops, where WiFi would pretty much be guaranteed?</p>
<p>Share your thoughts in the comments.</p>
<p><em>Header image courtesy of <a href="https://flic.kr/p/7vuxbx">Andy Rennie</a></em></p>
<a data-banner_id="21" data-campaign_id="11" href="http://www.telerik.com/platform" target="_blank" class="acc_banner_link "><img src="http://developer.telerik.com/wp-content/uploads/ac_uploads/1177154.jpg" width="570" height="100"/></a>
